/*  gtktilink - link program for TI calculators
 *  Copyright (C) 1999, 2000  Romain Lievin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>

#include "indep.h"

/*
  Check whether the calc is ready
*/
int cb_calc_is_ready(void)
{
  if(is_active) 
    return -1;
  
  if(error(ti_calc.isready()))
    return 1;

  return 0;
}

/*
  Send a backup
*/
int cb_send_backup(char *filename)
{
  int err;
  FILE *bck;
  int file_mode = MODE_NORMAL;

  if(is_active)
    return 0;

  if(options.file_mode == EXTENDED_FORMAT)
    file_mode=MODE_KEEP_ARCH_ATTRIB;

  switch(options.lp.calc_type)
    {
    case CALC_TI82:
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI85:
    case CALC_TI92:
      create_pbar_type3(gettext("Backup"));
      break;
    case CALC_TI89:
    case CALC_TI92P:
      create_pbar_type5(gettext("Backup"), "");
      break;
      break;
    }

  open_ti_file(filename, "rb", &bck);
  err=ti_calc.send_backup(bck, file_mode);
  close_ti_file();
  switch(options.lp.calc_type)
    {
    case CALC_TI82:
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI85:
    case CALC_TI92:
      destroy_pbar_type3();
      break;
    case CALC_TI89:
    case CALC_TI92P:
      destroy_pbar_type5();
      break;
    }
  if(error(err)) return 1;

  return 0;
}

/*
  Receive a backup
*/
int cb_receive_backup(void)
{
  FILE *bck;
  int err=0;
  longword version;
  char tmp_filename[MAXCHARS];
  int file_mode=MODE_NORMAL;
  //clock_t start, stop; // for benchmark test


  if(is_active) 
    return -1;
  
  if(options.file_mode == EXTENDED_FORMAT)
    file_mode=MODE_KEEP_ARCH_ATTRIB;
  
  switch(options.lp.calc_type)
    {
    case CALC_TI82:
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI85:
      create_pbar_type3(gettext("Backup"));
      break;
    case CALC_TI89:
    case CALC_TI92P:
      create_pbar_type5(gettext("Backup"), "");
      break;
    case CALC_TI92:
      create_pbar_type2(gettext("Backup"), gettext("Receiving blocks"));
      break;
    } 

  strcpy(tmp_filename, gu_get_tmp_dir());
  strcat(tmp_filename, DIR_SEPARATOR);
  strcat(tmp_filename, "gtktilink.backup");
  do
    {
      refresh_gui();
      if(info_update.cancel) break;

      open_ti_file(tmp_filename, "wb", &bck);
      //start=clock();
      err=ti_calc.receive_backup(bck, MODE_NORMAL | file_mode, &version);
      //stop=clock();
      //fprintf(stdout, "Time: %u %f\n", stop-start, (float)(stop-start)/CLOCKS_PER_SEC);
      close_ti_file(bck);
    }
  while( ((err == 35) || (err == 3)) && ((options.lp.calc_type == CALC_TI82) 
					 || (options.lp.calc_type == CALC_TI85)) );
  
  switch(options.lp.calc_type)
    {
    case CALC_TI82:
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI85:
      destroy_pbar_type3();
      break;
    case CALC_TI89:
    case CALC_TI92P:
      destroy_pbar_type5();
      break;
    case CALC_TI92:
      destroy_pbar_type2();
      break;
    }
  if(error(err)) 
    return 1;

  return 0;
}

/*
  Receive the IDlist
*/
int cb_id_list(void)
{
  char buffer[MAXCHARS];
  char idlist[MAXCHARS];
  int err;

  if(is_active) 
    return -1;

  err=ti_calc.receive_var(NULL, MODE_IDLIST, idlist, 0, 0);
  if(error(err)) 
    return 0;
  strcpy(buffer, "IDlist: ");
  strcat(buffer, idlist+2);
  msg_box(gettext("IDlist"), 
	  buffer);

  return 1;
}

/*
  Do a ROM dump
*/
int cb_rom_dump(void)
{
	if(is_active) 
    return -1;

  return 0;
}

/*
  Get the ROM version
*/
int cb_rom_version(void)
{
  int err;
  char version[MAXCHARS];
  char buffer[MAXCHARS];
  
  err = ti_calc.get_rom_version(version);
  if(error(err))
    {
      return 1; 
    }
  sprintf(buffer, "ROM version %s\n", version);
  user1_box(gettext("ROM version"), buffer, gettext(" OK "));

  return 0;
}

/*
  Send one or more selected variables
*/
int cb_send_var(void)
{
  GList *ptr;
  FILE *txt;
  struct file_info *f=NULL;
  gint i=0;
  gint l=0;
  int path_mode=MODE_NORMAL;
  int file_mode=MODE_NORMAL;
  
  if(is_active)
    return -1;
  if(clist_win.selection == NULL) 
    return 0;

  if(options.path_mode == LOCAL_PATH)
    path_mode=MODE_LOCAL_PATH;
  if(options.file_mode == EXTENDED_FORMAT)
    file_mode=MODE_KEEP_ARCH_ATTRIB;

  ptr=clist_win.selection;
  l=gu_list_length(clist_win.selection);
  
  /* Choose the appropriate dialog box */
  if(l == 1)
    {
      f=(struct file_info *)ptr->data;
      if(strstr(f->filename, ti_calc.group_file_ext(options.lp.calc_type)))
	create_pbar_type5(gettext("Sending group file"), "");
      else
	create_pbar_type4(gettext("Sending variable"), "");
    }
  else
    create_pbar_type5(gettext("Sending variables"), "");
  
  /* Send the files */
  while(ptr != NULL)
    {
      f=(struct file_info *)ptr->data;
      
      if(strstr(f->filename, ti_calc.group_file_ext(options.lp.calc_type)))
	{ 
	  /****************/
	  /* A group file */
	  /****************/
	  
	  open_ti_file(f->filename, "rb", &txt);
	  
	  /* It is not the last file to send */
	  if( ((ptr->next) != NULL) && (l > 1) )
	    {
	      /* More than one file to send */
	      if(error(ti_calc.send_var(txt, MODE_SEND_VARS 
					| path_mode | file_mode)))
		{
		  destroy_pbar();
		  return 1;
		}
	    }
	  else
	    {
	      /* It is the last one */
	      if(error(ti_calc.send_var(txt, MODE_SEND_LAST_VAR 
					| path_mode | file_mode)))
		{
		  destroy_pbar();
		  return 1;
		}
	    }
	  
	}
      else
	{ 
	  /********************/
	  /* Not a group file */
	  /********************/
	  
	  if(strstr(f->filename, ti_calc.backup_file_ext(options.lp.calc_type)))
	    {
	      error(21);
	      destroy_pbar();
	      return 1;
	    }
	  open_ti_file(f->filename, "rb", &txt);

	  /* It is not the last file to send */
	  if( ((ptr->next) != NULL) && (l > 1) )
	    {
	      /* More than one file to send */
	      if(error(ti_calc.send_var(txt, MODE_SEND_VARS 
					| path_mode | file_mode)))
		{
		  destroy_pbar();
		  return 1;
		}
	    }
	  else
	    {
	      /* There is one */
	      if(error(ti_calc.send_var(txt, MODE_SEND_ONE_VAR 
					| path_mode | file_mode)))
		{
		  destroy_pbar();
		  return 1;
		}
	    }
	  close_ti_file();
	}
      
      i++;
      if(l > 1)
	{
	  info_update.main_percentage=(float)i/l;
	  refresh_pbar();
	  refresh_gui();
	}
      
      ptr=ptr->next;
    }
  destroy_pbar();

  return 0;
}

/*
  Receive one or more selected variables
*/
int cb_receive_var(int *to_save)
{
  int err=0;
  GList *ptr;
  FILE *txt;
  struct varinfo *v;
  char var_n[20]; /* 8+1+8 characters max */
  char file_n[25];
  int i,l;
  char str[16];
  char tmp_filename[MAXCHARS];
  int ret, skip=0;
  char buffer[MAXCHARS];
  char *dirname;
  char varname[9];
  int file_mode=MODE_NORMAL;

  *to_save = 0;

  if(is_active) 
    return -1;
  
  if(options.file_mode == EXTENDED_FORMAT)
    file_mode=MODE_KEEP_ARCH_ATTRIB;
  
  switch(options.lp.calc_type)
    {
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI86:
    case CALC_TI89:
    case CALC_TI92:
    case CALC_TI92P:
      if(ctree_win.selection==NULL) 
	return 0;
      if(gu_list_length(ctree_win.selection)==1)
	{
	  /* Single file */
	  create_pbar_type4(gettext("Receiving variable"), "");
	  v=(struct varinfo *)ctree_win.selection->data;
	  
	  if( (options.lp.calc_type != CALC_TI83) && (options.lp.calc_type != CALC_TI83P) 
	      && (options.lp.calc_type != CALC_TI86) )
	    {
	      strcpy(var_n, (v->folder)->varname);
	      strcat(var_n, "\\");
	      strcat(var_n, v->varname);
	    }
	  else
	    {
	      strncpy(var_n, v->varname, 9);
	    }
	  strcpy(file_n, v->translate);
	  strcat(file_n, ".");
	  strcat(file_n, ti_calc.byte2fext(v->vartype));
	  
	  if(options.confirm == CONFIRM_YES)
	    {
	      if( access(file_n, F_OK) == 0 )
		{
		  sprintf(buffer, gettext("The file %s already exists."),
			  file_n);
		  ret=user3_box(gettext("Warning"), buffer,
				gettext(" Overwrite "), gettext(" Rename "),
				gettext(" Skip "));
		  switch(ret)
		    {
		    case BUTTON2:
		      dirname=dlgbox_entry(gettext("Rename the file"),
					   gettext("New name: "), file_n);
		      if(dirname == NULL) 
			return 1;
		      strcpy(file_n, dirname);
		      gu_free(dirname);
		    case BUTTON1:
		      skip=0;
		      break;
		    case BUTTON3:
		      skip=1;
		      break;
		    default:
		      break;
		    }
		}
	    }
	  if(skip == 0)
	    {
	      open_ti_file(file_n, "wb", &txt);
	     
	      /*
		printf("%8s %8s %8s %02X\n", (v->folder)->varname, v->varname, v->translate, v->vartype);
		printf("-> %p\n", txt);
	      */
	      /* This part generates the header for single files */
	      switch(options.lp.calc_type)
		{
		case CALC_TI89:
		  ti_calc.generate_single_file_header(txt, file_mode, 
						      "**TI92P*", 
						      v);
		  break;
		case CALC_TI92:
		  ti_calc.generate_single_file_header(txt, file_mode, 
						      "**TI92**", v);
		  break;
		case CALC_TI92P:
		  ti_calc.generate_single_file_header(txt, file_mode, 
						      "**TI92P*", v);
		  break;
		default:
		  break;
		}
	      err=ti_calc.receive_var(txt, MODE_RECEIVE_SINGLE_VAR 
				      | file_mode, 
				      var_n, v->vartype, v->varlocked);
	      close_ti_file();
	      //fclose(txt);     
	      if(error(err))
		{
		  destroy_pbar_type1();
		  return 1; 
		}
	    }
	  destroy_pbar_type1();
	}
      else
	{
	  /* Group file */
	  create_pbar_type5(gettext("Receiving variables"), "");
	  strcpy(tmp_filename, gu_get_tmp_dir());
	  strcat(tmp_filename, DIR_SEPARATOR);
	  strcat(tmp_filename, "gtktilink.PAK");
	  open_ti_file(tmp_filename, "wb", &txt);
	  
	  switch(options.lp.calc_type)
	    {
	    case CALC_TI89:
	      generate_group_file_header(txt, file_mode, 
					 "**TI92P*", NULL,
					 options.lp.calc_type);
	      break;
	    case CALC_TI92:
	      generate_group_file_header(txt, file_mode, 
					 "**TI92**", NULL,
					 options.lp.calc_type);
	      break;
	    case CALC_TI92P:
	      generate_group_file_header(txt, file_mode, 
					 "**TI92P*", NULL,
					 options.lp.calc_type);
	      break;
	    }
	  
	  l=gu_list_length(ctree_win.selection);
	  i=0;
	  ptr=ctree_win.selection;
	  while(ptr!=NULL)
	    {
	      v=(struct varinfo *)ptr->data;
	      //printf("Varname: %s, vartype: %s\n", v->varname, ti_calc.byte2type(v->vartype));
	      //printf("Parent folder: %s\n", (v->folder)->varname);
	      if( (options.lp.calc_type != CALC_TI83) && (options.lp.calc_type != CALC_TI83P) && (options.lp.calc_type != CALC_TI86) )
		{
		  strcpy(var_n, (v->folder)->varname);
		  strcat(var_n, "\\");
		  strcat(var_n, v->varname);
		}
	      else
		{
		  strncpy(var_n, v->varname, 9);
		}
	      if(i == 0)
		err=ti_calc.receive_var(txt, MODE_RECEIVE_FIRST_VAR 
					| file_mode, 
					var_n, v->vartype, v->varlocked);
	      else if( i == l-1)
		err=ti_calc.receive_var(txt, MODE_RECEIVE_LAST_VAR 
					| file_mode, 
					var_n, v->vartype, v->varlocked);
	      else 
		err=ti_calc.receive_var(txt, MODE_RECEIVE_VARS | file_mode, 
				     var_n, v->vartype, v->varlocked);
	      if(error(err)) 
		{
		close_ti_file();
		  //fclose(txt);
		  destroy_pbar_type3();
		  return 1;
		}
	      i++;
	      info_update.main_percentage=(float)i/l;
#ifdef GTK
	      refresh_pbar();
	      refresh_gui();
#endif
	      ptr=ptr->next;
	    }      
	  //fclose(txt);
	  close_ti_file();
	  destroy_pbar_type3();
	  *to_save = 1;
	}
      break;
    case CALC_TI82:
    case CALC_TI85:
      create_pbar_type4(gettext("Receiving variable(s)"), "");
      strcpy(tmp_filename, gu_get_tmp_dir());
      strcat(tmp_filename, DIR_SEPARATOR);
      strcat(tmp_filename, "gtktilink.PAK");
      do
	{
	  open_ti_file(tmp_filename, "wb", &txt);
	
	  while( gtk_events_pending() ) { gtk_main_iteration(); }
	  if(info_update.cancel) break;
	  err=ti_calc.receive_var(txt, MODE_NORMAL | file_mode, varname, 0, 0);
	  //fclose(txt);
	  close_ti_file();
	}
      while((err == 35) || (err == 3));
      destroy_pbar_type4();
      if(error(err))
	{
	  return 1;
	}      
      
      if(varname[0] != '\0')
	{
	  /* Single file */
	  /* Some varnames should be translated */
	  if(options.lp.calc_type == CALC_TI82)
	    ti_calc.translate_varname(varname, str, 0);
	  else
	    strcpy(str, varname);
	  strcat(str, ".");
	  open_ti_file(tmp_filename, "rb", &txt);
	
	  for(i=0; i<11; i++) fgetc(txt);
	  for(i=0; i<42; i++) fgetc(txt);
	  for(i=0; i<6; i++) fgetc(txt);
	  strcat(str, ti_calc.byte2fext((byte)fgetc(txt)));
	  //fclose(txt);
	  close_ti_file();
	  
	  if(options.confirm == CONFIRM_YES)
	    {
	      if( access(str, F_OK) == 0 )
		{
		  sprintf(buffer, gettext("The file %s already exists."),
			  str);
		  ret=user3_box(gettext("Warning"), buffer,
				gettext(" Overwrite "), gettext(" Rename "),
				gettext(" Skip "));
		  switch(ret)
		    {
		    case BUTTON2:
		      dirname=dlgbox_entry(gettext("Rename the file"),
					   gettext("New name: "), str);
		      if(dirname == NULL) 
			return 1;
		      strcpy(str, dirname);
		      gu_free(dirname);
		    case BUTTON1:
		      skip=0;
		      break;
		    case BUTTON3:
		      skip=1;
		      break;
		    default:
		      break;
		    }
		}
	    }	      
	  
	  if(skip == 0)
	    {
	      move_file(tmp_filename, str);  
	    }
	}
      else
	{
	  /* Group file */
	  *to_save = 1;
	}
      break;
    }

  return 0;
}

int cb_change_drive(char drive_letter)
{
#ifdef __WIN32__
  char buffer[MAXCHARS];
  
  //printf("Drive: <%c:>\n", (char)drive_letter);
  //_chdrive( (int)drive_letter );
  gu_snprintf(clist_win.cur_dir, 8, "%c:\\", (char)drive_letter);
  _chdir(clist_win.cur_dir);
  //printf("New path: <%s>\n", clist_win.cur_dir);
#else
  msg_box(gettext("Information"),
	  gettext("This function is not available in the Win version."));
#endif

  return 0;
}

int cb_dirlist(void)
{
  if(is_active)
    return -1;

  c_directory_list();

  return 0;
}

/*
  Send a FLASH app
*/
int cb_send_flash_app(char *filename)
{
  int err;
  FILE *bck;
  gint old_timeout;

  if(is_active)
    return 0;

  switch(options.lp.calc_type)
    {
    case CALC_TI82:
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI85:
    case CALC_TI92:
      create_pbar_type3(gettext("Flash"));
      break;
    case CALC_TI89:
    case CALC_TI92P:
      create_pbar_type5(gettext("Flash"), "");
      break;
      break;
    }

  open_ti_file(filename, "rb", &bck);
  old_timeout = get_timeout();
  set_timeout(100); // 10 seconds
  err=ti_calc.send_flash(bck, MODE_APPS);
  set_timeout(old_timeout);
  close_ti_file();
  switch(options.lp.calc_type)
    {
    case CALC_TI82:
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI85:
    case CALC_TI92:
      destroy_pbar_type3();
      break;
    case CALC_TI89:
    case CALC_TI92P:
      destroy_pbar_type5();
      break;
    }
  if(error(err)) return 1;

  return 0;
}

/*
  Send a FLASH OS (AMS)
*/
int cb_send_flash_os(char *filename)
{
  int err;
  FILE *bck;
  gint old_timeout;

  if(is_active)
    return 0;

  switch(options.lp.calc_type)
    {
    case CALC_TI82:
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI85:
    case CALC_TI92:
      create_pbar_type3(gettext("Flash"));
      break;
    case CALC_TI89:
    case CALC_TI92P:
      create_pbar_type5(gettext("Flash"), "");
      break;
      break;
    }

  open_ti_file(filename, "rb", &bck);
  old_timeout = get_timeout();
  set_timeout(100); // 10 seconds
  err=ti_calc.send_flash(bck, MODE_AMS);
  set_timeout(old_timeout);
  close_ti_file();
  switch(options.lp.calc_type)
    {
    case CALC_TI82:
    case CALC_TI83:
    case CALC_TI83P:
    case CALC_TI85:
    case CALC_TI92:
      destroy_pbar_type3();
      break;
    case CALC_TI89:
    case CALC_TI92P:
      destroy_pbar_type5();
      break;
    }
  if(error(err)) return 1;

  return 0;
}
